/*
Copyright 2012  Vesko Georgiev vesko.m.georgiev@gmail.com

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either 
version 3 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public 
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.veskogeorgiev.utils.jdbc;

import java.util.LinkedList;
import java.util.List;

public class SelectStatement {

	private String tableName;
	private String tableNameAs;
	private String columns;
	private boolean distinct;
	private List<String> joinTableTerms = new LinkedList<String>();
	private Clause whereStatement;
	private String order;

	public static interface Clause {
		boolean isEmpty();
	}

	public static class SimpleClause implements Clause {
		private String str;

		public SimpleClause(String str) {
			this.str = str;
		}

		@Override
		public String toString() {
			return str;
		}

		@Override
		public boolean isEmpty() {
			return str == null;
		}
	}

	public static abstract class CompositeClause implements Clause {
		private List<Clause> clauses = new LinkedList<Clause>();
		private String junction;

		protected CompositeClause(String junction) {
			this.junction = junction;
		}

		public void add(Clause c) {
			clauses.add(c);
		}

		public void add(String c) {
			clauses.add(new SimpleClause(c));
		}

		@Override
		public boolean isEmpty() {
			return clauses.isEmpty();
		}

		@Override
		public String toString() {
			StringBuilder builder = new StringBuilder();
			for (Clause c : clauses) {
				builder.append(c.toString()).append(" ").append(junction).append(" ");
			}
			if (builder.length() > 0) {
				int idx = builder.length() - junction.length() - 2;
				builder.delete(idx, idx + junction.length() + 2);
				builder.insert(0, "(");
				builder.append(")");
			}
			return builder.toString();
		}
	}

	public static class AndsClause extends CompositeClause {
		public AndsClause() {
			super("and");
		}
	}

	public static class OrsClause extends CompositeClause {
		public OrsClause() {
			super("or");
		}
	}

	public SelectStatement from(String tableName, String as) {
		this.tableName = tableName;
		this.tableNameAs = as;
		return this;
	}

	public SelectStatement from(String tableName) {
		return from(tableName, null);
	}

	public SelectStatement join(String table, String as, String param1, String param2) {
		StringBuilder builder = new StringBuilder();
		if (as == null) {
			as = table;
		}
		builder.append(" join ").append(table).append(" as ").append(as).append(" on ");
		builder.append(tableName).append(".").append(param1).append(" = ").append(as).append(".").append(param2);
		joinTableTerms.add(builder.toString());
		return this;
	}

	public SelectStatement join(String table, String param1, String param2) {
		return join(table, null, param1, param2);
	}

	public SelectStatement columns(String columns) {
		this.columns = columns;
		return this;
	}

	public SelectStatement addColumn(String columns) {
		if (columns == null || columns.isEmpty()) {
			this.columns = columns;
		}
		else {
			this.columns += ", " + columns;
		}
		return this;
	}

	public SelectStatement distinct(boolean distinct) {
		this.distinct = distinct;
		return this;
	}

	public SelectStatement where(Clause clause) {
		if (!clause.isEmpty()) {
			whereStatement = clause;
		}
		return this;
	}

	public SelectStatement orderBy(String key, boolean asc) {
		this.order = "order by " + key + " " + (asc ? "asc" : "desc");
		return this;
	}

	StringBuilder builder;

	@Override
	public String toString() {
		builder = new StringBuilder();
		append("select");
		if (distinct) {
			append("distinct");
		}
		append(columns).append("from").append(tableName);
		if (tableNameAs != null) {
			append("as").append(tableNameAs);
		}
		for (String join : joinTableTerms) {
			append(join);
		}
		if (whereStatement != null) {
			append("where " + whereStatement);
		}
		if (order != null) {
			append(order);
		}
		return builder.toString();
	}

	private SelectStatement append(String s) {
		builder.append(" ").append(s).append(" ");
		return this;
	}

}
